/*
  $Id: flags.h 664 2006-01-19 09:46:45Z jens $

  Copyright (C) 2001-2006 Jens Thoms Toerring
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  To contact the author send email to: jt@toerring.de
*/


/*
  This file is also included by the user land library so we need to include
  the macros from ioctl.h to make the library understand the command numbers
*/


#include <linux/ioctl.h>


typedef struct {
	unsigned long   cnt;        /* I/O count              */
	int             arg;        /* other arguments        */
	void          * buf;        /* I/O buffer             */
	int             stat;       /* returned status        */
	int             err;        /* returned error code    */
} GPIB_Arg;


/* Definition the numbers of all the different ioctl() commands - the
   character 'j' as the 'magic' type seems to be not in use for command
   numbers above 0x3F (see linux/Documentation/ioctl-number.txt). All
   commands need write access to the sructure passed to ioctl() because
   status and error states are returned via this structure. */

#define GPIB_MAGIC_IOC      'j'

#define GPIB_IOC_READ       _IOWR( GPIB_MAGIC_IOC, 0xC0, GPIB_Arg )
#define GPIB_IOC_WRT        _IOWR( GPIB_MAGIC_IOC, 0xC1, GPIB_Arg )
#define GPIB_IOC_CMD        _IOWR( GPIB_MAGIC_IOC, 0xC2, GPIB_Arg )
#define GPIB_IOC_DVRD       _IOWR( GPIB_MAGIC_IOC, 0xC3, GPIB_Arg )
#define GPIB_IOC_DVWRT      _IOWR( GPIB_MAGIC_IOC, 0xC4, GPIB_Arg )
#define GPIB_IOC_WAIT       _IOWR( GPIB_MAGIC_IOC, 0xC5, GPIB_Arg )
#define GPIB_IOC_ONL        _IOR(  GPIB_MAGIC_IOC, 0xC6, GPIB_Arg )
#define GPIB_IOC_SIC        _IOR(  GPIB_MAGIC_IOC, 0xC7, GPIB_Arg )
#define GPIB_IOC_REN        _IOWR( GPIB_MAGIC_IOC, 0xC8, GPIB_Arg )
#define GPIB_IOC_CAC        _IOR(  GPIB_MAGIC_IOC, 0xCA, GPIB_Arg )
#define GPIB_IOC_GTS        _IOR(  GPIB_MAGIC_IOC, 0xCB, GPIB_Arg )
#define GPIB_IOC_TRG        _IOWR( GPIB_MAGIC_IOC, 0xCC, GPIB_Arg )
#define GPIB_IOC_SDC        _IOWR( GPIB_MAGIC_IOC, 0xCD, GPIB_Arg )
#define GPIB_IOC_DCL        _IOR(  GPIB_MAGIC_IOC, 0xCE, GPIB_Arg )
#define GPIB_IOC_LOCAL      _IOWR( GPIB_MAGIC_IOC, 0xCF, GPIB_Arg )
#define GPIB_IOC_RSP        _IOWR( GPIB_MAGIC_IOC, 0xD0, GPIB_Arg )
#define GPIB_IOC_PPC        _IOWR( GPIB_MAGIC_IOC, 0xD1, GPIB_Arg )
#define GPIB_IOC_RPP        _IOWR( GPIB_MAGIC_IOC, 0xD2, GPIB_Arg )
#define GPIB_IOC_RSV        _IOR(  GPIB_MAGIC_IOC, 0xD3, GPIB_Arg )
#define GPIB_IOC_TIMO       _IOWR( GPIB_MAGIC_IOC, 0xD4, GPIB_Arg )
#define GPIB_IOC_EOS        _IOWR( GPIB_MAGIC_IOC, 0xD5, GPIB_Arg )
#define GPIB_IOC_EOT        _IOWR( GPIB_MAGIC_IOC, 0xD6, GPIB_Arg )
#define GPIB_IOC_DMA        _IOWR( GPIB_MAGIC_IOC, 0xD7, GPIB_Arg )
#define GPIB_IOC_IRQ        _IOWR( GPIB_MAGIC_IOC, 0xD8, GPIB_Arg )
#define GPIB_IOC_LINES      _IOWR( GPIB_MAGIC_IOC, 0xD9, GPIB_Arg )
#define GPIB_IOC_PCT        _IOR(  GPIB_MAGIC_IOC, 0xDA, GPIB_Arg )
#define GPIB_IOC_PAD        _IOWR( GPIB_MAGIC_IOC, 0xDB, GPIB_Arg )
#define GPIB_IOC_SAD        _IOWR( GPIB_MAGIC_IOC, 0xDC, GPIB_Arg )
#define GPIB_IOC_STATE      _IOR(  GPIB_MAGIC_IOC, 0xDD, GPIB_Arg )
#define GPIB_IOC_LN         _IOWR( GPIB_MAGIC_IOC, 0xDE, GPIB_Arg )
#define GPIB_IOC_LLO        _IOWR( GPIB_MAGIC_IOC, 0xDF, GPIB_Arg )

#define GPIB_MIN_IOC_NR     _IOC_NR( GPIB_IOC_READ )
#define GPIB_MAX_IOC_NR     _IOC_NR( GPIB_IOC_LLO )


/* gpib_status bits (returned by all functions) */

#define GPIB_ERR    ( 1 << 15 )   /* Function call terminated on error       */
#define GPIB_TIMO   ( 1 << 14 )   /* Time limit on I/O or wait exceeded      */
#define GPIB_END    ( 1 << 13 )   /* EOI terminated the ibrd call            */
#define GPIB_SRQI   ( 1 << 12 )   /* SRQ is asserted                         */
#define GPIB_RQS    ( 1 << 11 )   /* Device requesting service               */
#define GPIB_CMPL   ( 1 <<  8 )   /* I/O is complete (should always be set)  */
#define GPIB_CIC    ( 1 <<  5 )   /* GPIB interface is Controller-in-Charge  */
#define GPIB_ATN    ( 1 <<  4 )   /* Attention is asserted                   */
#define GPIB_TACS   ( 1 <<  3 )   /* GPIB interface is addressed as Talker   */
#define GPIB_LACS   ( 1 <<  2 )   /* GPIB interface is addressed as Listener */
#define GPIB_DTAS   ( 1 <<  1 )   /* Device trigger state                    */
#define GPIB_DCAS   ( 1 <<  0 )   /* Device clear state                      */

#define GPIB_ACIC   ( GPIB_CIC | GPIB_ATN )


/* Valid bits to be used with gpib_wait() */

#define GPIB_WAIT_BITS \
                ( GPIB_TIMO | GPIB_SRQI | GPIB_CIC | GPIB_TACS | GPIB_LACS )


/* GPIB_ERR error codes */

#define GPIB_EDVR     0	      /* system error                                */
#define GPIB_ECIC     1	      /* not CIC                                     */
#define GPIB_ENOL     2	      /* no listeners                                */
#define GPIB_EADR     3	      /* CIC and not addressed before I/O            */
#define GPIB_EARG     4	      /* bad argument to function call               */
#define GPIB_ESAC     5	      /* not SAC                                     */
#define GPIB_EABO     6	      /* I/O operation was aborted                   */
#define GPIB_ENEB     7	      /* non-existent board (GPIB interface offline) */
#define GPIB_EDMA     8	      /* DMA hardware error detected                 */
#define GPIB_EBTO     9	      /* DMA hardware uP bus timeout                 */
#define GPIB_EOIP    10	      /* new I/O attempted with old I/O in progress  */
#define GPIB_ECAP    11	      /* no capability for intended opeation         */
#define GPIB_EFSO    12	      /* file system operation error                 */
#define GPIB_EOWN    13	      /* shareable board exclusively owned           */
#define GPIB_EBUS    14	      /* bus error                                   */
#define GPIB_ESTB    15	      /* lost serial poll bytes                      */
#define GPIB_ESRQ    16	      /* SRQ stuck on                                */
#define GPIB_ECFG    17       /* Config file not found                       */
#define GPIB_EPAR    18       /* Parse Error in Config                       */
#define GPIB_ETAB    19       /* Table Overflow                              */
#define GPIB_ENSD    20       /* Device not found                            */
#define GPIB_ENWE    21       /* Network Error                               */
#define GPIB_ENTF    22       /* Nethandle-table full                        */
#define GPIB_EMEM    23       /* Memory allocation Error                     */


/* Timeout values */

#define TNONE       0		/* Infinite timeout (disabled)     */
#define T10us       1		/* Timeout of 10 usec              */
#define T30us       2		/* Timeout of 30 usec              */
#define T100us      3		/* Timeout of 100 usec             */
#define T300us      4		/* Timeout of 300 usec             */
#define T1ms        5		/* Timeout of 1 msec               */
#define T3ms        6		/* Timeout of 3 msec               */
#define T10ms       7		/* Timeout of 10 msec              */
#define T30ms       8		/* Timeout of 30 msec              */
#define T100ms      9		/* Timeout of 100 msec             */
#define T300ms     10		/* Timeout of 300 msec             */
#define T1s        11		/* Timeout of 1 sec                */
#define T3s        12		/* Timeout of 3 sec                */
#define T10s       13		/* Timeout of 10 sec               */
#define T30s       14		/* Timeout of 30 sec               */
#define T100s      15		/* Timeout of 100 sec              */
#define T300s      16		/* Timeout of 300 sec              */
#define T1000s     17		/* Timeout of 1000 sec (maximum)   */


/* End-of-string (EOS) modes for use with gpib_eos() */

#define GPIB_REOS     0x01	/* Terminate reads on EOS	  */
#define GPIB_XEOS     0x02	/* Set EOI with EOS on writes	  */
#define GPIB_BIN      0x04	/* Do 8-bit compare on EOS	  */
#define GPIB_EOT      0x08	/* Send END with last byte	  */

#define GPIB_EOSM	( GPIB_REOS | GPIB_XEOS | GPIB_BIN )


/* GPIB Bus Control Lines bit vector */

#define BusDAV	   ( 1 <<  8 )         /* DAV  line status bit */
#define BusNDAC	   ( 1 <<  9 )         /* NDAC line status bit */
#define BusNRFD	   ( 1 << 10 )         /* NRFD line status bit */
#define BusIFC	   ( 1 << 11 )         /* IFC  line status bit */
#define BusREN	   ( 1 << 12 )         /* REN  line status bit */
#define BusSRQ	   ( 1 << 13 )         /* SRQ  line status bit */
#define BusATN	   ( 1 << 14 )         /* ATN  line status bit */
#define BusEOI	   ( 1 << 15 )         /* EOI  line status bit */

#define ValidDAV   ( 1 <<  0 )
#define ValidNDAC  ( 1 <<  1 )
#define ValidNRFD  ( 1 <<  2 )
#define ValidIFC   ( 1 <<  3 )
#define ValidREN   ( 1 <<  4 )
#define ValidSRQ   ( 1 <<  5 )
#define ValidATN   ( 1 <<  6 )
#define ValidEOI   ( 1 <<  7 )


/* Possible GPIB command messages */

#define LAD	     0x20	/* 'or'ed listen address	*/
#define TAD	     0x40	/* 'or'ed talk address  	*/
#define GTL	     0x01	/* go to local			*/
#define SDC	     0x04	/* selected device clear 	*/
#define PPC	     0x05	/* parallel poll configure	*/
#define GET	     0x08	/* group execute trigger 	*/
#define TCT	     0x09	/* take control 		*/
#define LLO	     0x11	/* local lockout		*/
#define DCL	     0x14	/* device clear 		*/
#define PPU	     0x15	/* parallel poll unconfigure	*/
#define SPE	     0x18	/* serial poll enable	 	*/
#define SPD	     0x19	/* serial poll disable	 	*/
#define UNL	     0x3F	/* unlisten			*/
#define UNT	     0x5F	/* untalk 			*/
#define PPE	     0x60	/* parallel poll enable (or'ed) */
#define PPD	     0x70	/* parallel podd disable	*/
#define S	     0x08	/* parallel poll sense bit	*/



/*
 * Local variables:
 * c-basic-offset: 8
 * c-indent-level: 8
 * c-brace-imaginary-offset: 0
 * c-brace-offset: 0
 * c-argdecl-indent: 4
 * c-label-ofset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: t
 * tab-width: 8
 * End:
 */
