/*
  $Id: gpib_module.h 735 2006-06-11 13:50:16Z jens $

  Copyright (C) 2001-2006 Jens Thoms Toerring
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  To contact the author send email to: jt@toerring.de
*/

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION( 2, 6, 19 )
#include <linux/config.h>
#else
#include <linux/utsrelease.h>
#endif


#if LINUX_VERSION_CODE < KERNEL_VERSION( 2, 6, 0 )

#include <linux/autoconf.h>

#if defined( CONFIG_MODVERSIONS ) && ! defined( MODVERSIONS )
#define MODVERSIONS
#endif

/* Including modversions.h must come before including module.h !
   On some systems you get very strange compiler time errors otherwise
   (as I learned the hard way...) */

#if defined( MODVERSIONS )
#include <linux/modversions.h>
#endif

#define __NO_VERSION__
#include <linux/module.h>

#include <linux/version.h>

#if ! defined( KERNEL_VERSION )
#define KERNEL_VERSION( a, b, c ) ( ( ( a ) << 16 ) | ( ( b ) << 8 ) | ( c ) )
#endif

#else

#include <linux/module.h>

#endif

#include <linux/kernel.h>
#include <linux/fs.h>

#include <linux/stddef.h>             /* for definition of NULL              */
#include <linux/timer.h>              /* for definition of struct timer_list */
#include <asm/semaphore.h>            /* for definition of struct semaphore  */

#if LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 2, 0 )
#include <linux/init.h>
#include <asm/uaccess.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#else
#include <linux/mm.h>
#include <linux/malloc.h>
#endif

#include <linux/ioport.h>
#include <linux/sched.h>
#include <asm/irq.h>
#include <asm/dma.h>
#include <linux/delay.h>

#ifdef CONFIG_PROC_FS
#include <linux/proc_fs.h>
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 6, 0 )
#include <linux/moduleparam.h>
#include <linux/cdev.h>
#include <linux/device.h>
#else
#define __user
#endif


/* PORT is the type of the out- and input ports for outb() and inb()
   Take care: This is hardware dependent! */

#define PORT unsigned int

#include "autoconf.h"
#include "types.h"


/* Some definitions for printing of debugging information */

#undef PDEBUG
#undef PDEBUGin
#undef PDEBUGout
#undef PDEBUG_STRING

#if defined GPIB_DEBUG


#define PDEBUG_STRING "                                                  "
#define PDEBUGin( )                                                    \
	do {                                                           \
		printk( KERN_DEBUG "GPIB:%.*s\\ %s()\n", ++PDEBUG_len, \
			PDEBUG_STRING, __FUNCTION__ );                 \
	} while( 0 )
#define PDEBUGout( )                                                   \
	do {                                                           \
		printk( KERN_DEBUG "GPIB:%.*s/ %s()\n", PDEBUG_len--,  \
			PDEBUG_STRING, __FUNCTION__ );                 \
	} while( 0 )
#define PDEBUG( fmt, args... )                                         \
	do {                                                           \
		printk( KERN_DEBUG "GPIB:  %.*s " fmt, PDEBUG_len,     \
			PDEBUG_STRING , ##args );                      \
	} while( 0 )
#else

#define PDEBUGin( )
#define PDEBUGout( )
#define PDEBUG( ftm, args... )

#endif


/*
 * Local variables:
 * c-basic-offset: 8
 * c-indent-level: 8
 * c-brace-imaginary-offset: 0
 * c-brace-offset: 0
 * c-argdecl-indent: 4
 * c-label-ofset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: t
 * tab-width: 8
 * End:
 */
