/*
  $Id: types.h 735 2006-06-11 13:50:16Z jens $

  Copyright (C) 2001-2006 Jens Thoms Toerring
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  To contact the author send email to: jt@toerring.de
*/


#include "flags.h"

#if defined __KERNEL__       /* user land programs don't need to see this... */


/* Some lines must stay unchanged for at least 100 us - the following
   definition is this time in microseconds (actually, we wait a bit longer,
   this won't hurt but makes real sure we don't get below the 100 us limit */

#define GPIB_MIN_TIME 150


#if LINUX_VERSION_CODE < KERNEL_VERSION( 2, 6, 19 )
typedef enum
{
	false = 0,
	true  = 1
} bool;
#endif

#define UNSET   false
#define OFF     false
#define FAIL    false
#define DMA_RD  false

#define SET     true
#define ON      true
#define OK      true
#define DMA_WR  true


#define EXCL       ( unsigned int ) ( 1 << 0 )
#define USE_IRQ    ( unsigned int ) ( 1 << 1 )
#define USE_DMA    ( unsigned int ) ( 1 << 2 )
#define TIMER_ON   ( unsigned int ) ( 1 << 3 )    /* set while timer is on */
#define REN        ( unsigned int ) ( 1 << 4 )
#define HOLDOFF    ( unsigned int ) ( 1 << 5 )



/* Controller chip types */

#define PD7210      ( unsigned int ) ( 1 << 0 )
#define TNT4882     ( unsigned int ) ( 1 << 1 )


/* Board types */

#define PCII        ( ( 1 <<  2 ) | PD7210  )
#define PCIIA       ( ( 1 <<  3 ) | PD7210  )
#define ATGPIB      ( ( 1 <<  4 ) | TNT4882 )



typedef struct Transfer Transfer_t;
typedef struct Board Board_t;


struct Transfer {
	unsigned char *    buf;        /* pointer to buffer with date       */
	unsigned int       cnt;        /* number of bytes to transfer       */
	bool               is_last;    /* set on last transfers */
	bool               dir;        /* direction of transfer             */
	struct semaphore   sem;        /* semaphore used in transfers       */
};


struct Board {
	const char *    name;
		       
	bool            is_running;     /* set when initialized              */

#if LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 6, 0 )
	struct cdev     ch_dev;
	dev_t           dev_no;
#if LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 6, 14 )
	struct class *  gpib_class;
#endif
#endif
	int             major;          /* major device number               */
	void *          base;           /* address of first register         */
	unsigned long   io_size;        /* width of the I/O address range    */
	unsigned int    irq;	        /* interrupt number                  */
	unsigned int    dma;            /* DMA channel                       */
		       		 
	unsigned int    status;         /* current status of board           */
	unsigned int    open_count;	 
	bool            use_dma;        /* set if dma is to be used          */
	bool            use_irq;        /* set if interrupt is to be used    */
		       		 
	struct semaphore lock_sem;      /* semaphore for locking             */

	unsigned int    type;           /* type of board                     */
		       		 
	unsigned char   pad;            /* boards primary address            */
	unsigned char   sad;            /* boards secondary address          */
		       		 
	unsigned int    timer_index;    /* current timeout limit             */
	volatile bool   timeout;        /* set on timeouts                   */
	struct timer_list timer;        /* timer structure                   */
					 
	unsigned char * buf;            /* buffer for transfers              */
	unsigned long   buf_size;       /* size of buffer                    */

	unsigned char   eos;
	unsigned int    eosmode;
	unsigned char   eos_byte;
		       
	Transfer_t      transf;

	unsigned long   total_read,
		        open_read,
		        last_read,
		        total_write,
		        open_write,
		        last_write;

	int             last_error;
	unsigned int    gpib_status;
};

#endif


/*
 * Local variables:
 * c-basic-offset: 8
 * c-indent-level: 8
 * c-brace-imaginary-offset: 0
 * c-brace-offset: 0
 * c-argdecl-indent: 4
 * c-label-ofset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: t
 * tab-width: 8
 * End:
 */
