/*
  $Id: dma.c 714 2006-03-07 10:21:18Z jens $

  Copyright (C) 2001-2006 Jens Thoms Toerring
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  To contact the author send email to: jt@toerring.de
*/


#include "gpib_module.h"
#include "sys_common.h"
#include "sys.h"
#include "board.h"



/*-----------------------------------------------------------------------*
 * Switches use of DMA on or off depending on the argument. DMA can't be
 * switched on if it wasn't compiled in, interrupts can't or shouldn't
 * be used or the DMA channel couldn't be requested.
 *-----------------------------------------------------------------------*/

int gpib_set_dma( int state )
{
	PDEBUGin( );

	if ( state != 0 ) {                      /* switch DMA on */
		if ( board.use_dma && board.use_irq && board.status & USE_IRQ )
			board.status |= USE_DMA;
		else {                           /* can't switch DMA on */
			PDEBUG( "Can't switch on use of DMA\n" );
			gpib_status |= GPIB_ERR;
			gpib_error   = GPIB_ECAP;
			PDEBUGout( );
			return - EINVAL;
		}
	} else                                   /* switch DMA off */
		board.status &= ~ USE_DMA;

	PDEBUGout( );
	return 0;
}


/*----------------------------------------------*
 * Function for enabling ISA DMA data transfers
 *----------------------------------------------*/

void gpib_isa_dma_enable( void *       buf,
			  unsigned int size,
			  bool         dir )
{
	unsigned long flags;


	PDEBUGin( );

#if LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 2, 0 )
	flags = claim_dma_lock( );
#else
	save_flags( flags );
	cli( );
#endif

	disable_dma( board.dma );
	clear_dma_ff( board.dma );

	set_dma_mode( board.dma,
		      dir == WRITE ? DMA_MODE_WRITE : DMA_MODE_READ );
	set_dma_addr( board.dma, virt_to_bus( buf ) );
	set_dma_count( board.dma, size );

	enable_dma( board.dma );

#if LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 2, 0 )
	release_dma_lock( flags );
#else
	restore_flags( flags );
#endif

	PDEBUGout( );
}


/*-----------------------------------------------*
 * Function for disabling ISA DMA data transfers
 *-----------------------------------------------*/

int gpib_isa_dma_disable( void )
{
	unsigned long flags;
	int residue;


	PDEBUGin( );

#if LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 2, 0 )
	flags = claim_dma_lock( );
#else
	save_flags( flags );
	cli( );
#endif

	disable_dma( board.dma );
	clear_dma_ff( board.dma );
	residue = get_dma_residue( board.dma );

#if LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 2, 0 )
	release_dma_lock( flags );
#else
	restore_flags( flags );
#endif

	PDEBUGout( );
	return residue;
}


/*
 * Local variables:
 * c-basic-offset: 8
 * c-indent-level: 8
 * c-brace-imaginary-offset: 0
 * c-brace-offset: 0
 * c-argdecl-indent: 4
 * c-label-ofset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: t
 * tab-width: 8
 * End:
 */
