/*
  $Id: gpib.h 736 2006-07-07 15:48:51Z jens $

  Copyright (C) 2001-2006 Jens Thoms Toerring
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  To contact the author send email to: jt@toerring.de
*/


#if ! defined GPIB_LIB_MAIN


#ifdef __cplusplus
extern "C" {
#endif


extern int gpib_status;
extern int gpib_error;
extern unsigned long gpib_count;

#endif


void gpib_error_setup( char * buffer,
                       size_t buffer_length );
int gpib_find( const char * dev_name );
int gpib_wrt( int          dev_dscr,
              const char * buffer,
              size_t       count );
int gpib_rd( int    dev_dscr,
             char * buffer,
             size_t count );
int gpib_cmd( const char * buffer,
              size_t       count );
int gpib_ask( int   dev_dscr,
              int   option,
              int * value );
int gpib_tmo( int dev_dscr,
              int timo );
int gpib_pad( int dev_dscr,
              int pad );
int gpib_sad( int dev_dscr,
              int sad );
int gpib_dev( int pad,
              int sad,
              int timo,
              int eos_char,
              int eos_mode );
int gpib_loc( int dev_dscr );
int gpib_llo( int dev_dscr );
int gpib_trg( int dev_dscr );
int gpib_clr( int dev_dscr );
int gpib_rsp( int             dev_dscr,
              unsigned char * c );
int gpib_eos( int dev_dscr,
              int eos_byte );
int gpib_onl( int dev_dscr,
              int v );
int gpib_cac( int v );
int gpib_sre( int v );
int gpib_gts( void );
int gpib_dcl( void );
int gpib_irq( int state );
int gpib_dma( int state );
int gpib_lines( int * lines );
int gpib_sic( void );
int gpib_eos_char( int dev_dscr,
                   int eos_char );
int gpib_eos_mode( int dev_dscr,
                   int eos_mode );
int gpib_ln( int         pad,
             int         sad,
             short int * listener );
int gpib_rpp( char * ppr );
int gpib_ppc( int dev_dscr,
              int v );


/* gpib_status bits (returned by all functions) */

#define GPIB_ERR    ( 1 << 15 )   /* Function call terminated on error       */
#define GPIB_TIMO   ( 1 << 14 )   /* Time limit on I/O or wait exceeded      */
#define GPIB_END    ( 1 << 13 )   /* EOI terminated the ibrd call            */
#define GPIB_SRQI   ( 1 << 12 )   /* SRQ is asserted                         */
#define GPIB_RQS    ( 1 << 11 )   /* Device requesting service               */
#define GPIB_CMPL   ( 1 <<  8 )   /* I/O is complete (should always be set)  */
#define GPIB_CIC    ( 1 <<  5 )   /* GPIB interface is Controller-in-Charge  */
#define GPIB_ATN    ( 1 <<  4 )   /* Attention is asserted                   */
#define GPIB_TACS   ( 1 <<  3 )   /* GPIB interface is addressed as Talker   */
#define GPIB_LACS   ( 1 <<  2 )   /* GPIB interface is addressed as Listener */
#define GPIB_DTAS   ( 1 <<  1 )   /* Device trigger state                    */
#define GPIB_DCAS   ( 1 <<  0 )   /* Device clear state                      */

/* GPIB_ERR error codes */

#define GPIB_EDVR       0     /* System error                                */
#define GPIB_ECIC       1     /* Not CIC                                     */
#define GPIB_ENOL       2     /* No listeners                                */
#define GPIB_EADR       3     /* CIC and not addressed before I/O            */
#define GPIB_EARG       4     /* Bad argument(s) to function call            */
#define GPIB_ESAC       5     /* Not SAC                                     */
#define GPIB_EABO       6     /* I/O operation was aborted                   */
#define GPIB_ENEB       7     /* Non-existent board                          */
#define GPIB_EDMA       8     /* DMA hardware error detected                 */
#define GPIB_EBTO       9     /* DMA hardware uP bus timeout                 */
#define GPIB_EOIP      10     /* New I/O attempted with old I/O in progress  */
#define GPIB_ECAP      11     /* No capability for intended opeation         */
#define GPIB_EFSO      12     /* File system operation error                 */
#define GPIB_EOWN      13     /* Shareable board exclusively owned           */
#define GPIB_EBUS      14     /* Bus error                                   */
#define GPIB_ESTB      15     /* Lost serial poll bytes                      */
#define GPIB_ESRQ      16     /* SRQ stuck on                                */
#define GPIB_ECFG      17     /* Config file not found                       */
#define GPIB_EPAR      18     /* Parse Error in Config                       */
#define GPIB_ETAB      19     /* Table Overflow                              */
#define GPIB_ENSD      20     /* Device not found                            */
#define GPIB_ENWE      21     /* Network Error                               */
#define GPIB_ENTF      22     /* Nethandle-table full                        */
#define GPIB_EMEM      23     /* Memory allocation Error                     */
#define GPIB_ENMS      24     /* No master device                            */
#define GPIB_ELCK      25     /* Device locked by other process              */


/* Timeout values */

#define TNONE           0     /* Infinite timeout (disabled)     */
#define T10us           1     /* Timeout of 10 usec              */
#define T30us           2     /* Timeout of 30 usec              */
#define T100us          3     /* Timeout of 100 usec             */
#define T300us          4     /* Timeout of 300 usec             */
#define T1ms            5     /* Timeout of 1 msec               */
#define T3ms            6     /* Timeout of 3 msec               */
#define T10ms           7     /* Timeout of 10 msec              */
#define T30ms           8     /* Timeout of 30 msec              */
#define T100ms          9     /* Timeout of 100 msec             */
#define T300ms         10     /* Timeout of 300 msec             */
#define T1s            11     /* Timeout of 1 sec                */
#define T3s            12     /* Timeout of 3 sec                */
#define T10s           13     /* Timeout of 10 sec               */
#define T30s           14     /* Timeout of 30 sec               */
#define T100s          15     /* Timeout of 100 sec              */
#define T300s          16     /* Timeout of 300 sec              */
#define T1000s         17     /* Timeout of 1000 sec (maximum)   */


/* End-of-string (EOS) modes for use with gpib_eos() */

#define GPIB_REOS     0x01    /* Terminate reads on EOS     */
#define GPIB_XEOS     0x02    /* Set EOI with EOS on writes */
#define GPIB_BIN      0x04    /* Do 8-bit compare on EOS    */
#define GPIB_EOT      0x08    /* Send END with last byte    */


/* Options to be used in gpib_ask() */

#define GPIB_ASK_SAD          1
#define GPIB_ASK_PAD          2
#define GPIB_ASK_EOS          3
#define GPIB_ASK_XEOS         4
#define GPIB_ASK_REOS         5
#define GPIB_ASK_BIN          6
#define GPIB_ASK_EOT          7
#define GPIB_ASK_TIMO         8
#define GPIB_ASK_IS_INIT      9
#define GPIB_ASK_IS_MASTER   10
#define GPIB_ASK_IRQ         11
#define GPIB_ASK_DMA         12


/* Defines of bits in the value returned by gpib_lines() */

#define ValidDAV   ( 1 <<  0 )
#define ValidNDAC  ( 1 <<  1 )
#define ValidNRFD  ( 1 <<  2 )
#define ValidIFC   ( 1 <<  3 )
#define ValidREN   ( 1 <<  4 )
#define ValidSRQ   ( 1 <<  5 )
#define ValidATN   ( 1 <<  6 )
#define ValidEOI   ( 1 <<  7 )

#define BusDAV     ( 1 <<  8 )
#define BusNDAC    ( 1 <<  9 )
#define BusNRFD    ( 1 << 10 )
#define BusIFC     ( 1 << 11 )
#define BusREN     ( 1 << 12 )
#define BusSRQ     ( 1 << 13 )
#define BusATN     ( 1 << 14 )
#define BusEOI     ( 1 << 15 )


#define NO_SAD      0
#define ALL_SAD    -1

#define PPE        0x60
#define PPD        0x70


#ifdef __cplusplus
} /* extern "C" */
#endif


/*
 * Local variables:
 * c-basic-offset: 8
 * c-indent-level: 8
 * c-brace-imaginary-offset: 0
 * c-brace-offset: 0
 * c-argdecl-indent: 4
 * c-label-ofset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
