/*
  $Id: parser.y 672 2006-01-19 12:11:17Z jens $

  Copyright (C) 2001-2006 Jens Thoms Toerring
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  To contact the author send email to: jt@toerring.de
*/


%{

#include "lib.h"


extern int gpib_dev_setup( Device * dev );
extern int yylex( void );

extern char *gpib_error_buffer;
extern size_t gpib_error_buffer_length;

void yyerror( const char * s );

static Device dev;
int dev_line;
extern int yylineno;

%}


%union
{
    bool bval;
    unsigned int ival;
    char *sval;
}


%token DEVICE_TOKEN NAME_TOKEN PAD_TOKEN SAD_TOKEN EOS_TOKEN
%token TIMO_TOKEN REOS_TOKEN XEOS_TOKEN BIN_TOKEN EOT_TOKEN
%token MASTER_TOKEN FILE_TOKEN ERR_TOKEN EN_TOKEN BOOL_TOKEN
%token TIME_TOKEN NUM_TOKEN STR_TOKEN NO_SAD_TOKEN

%type <bval> BOOL_TOKEN bool
%type <ival> NUM_TOKEN sad
%type <sval> STR_TOKEN

%%

line:   /* empty */
      | line device                         { if ( gpib_dev_setup( &dev ) < 0 )
                                                  return -1; }
      | error                               { return -1; }
      | ERR_TOKEN                           { return -1; }
;

device: DEVICE_TOKEN '{'                    { dev_line = yylineno;
		                                      *dev.name   = '\0';
                                              dev.pad     = -1;
                                              dev.sad     = NO_SAD;
                                              dev.eos     = '\n';
                                              dev.eosmode = GPIB_EOT;
                                              dev.flags   = 0;
                                              dev.timo    = GPIB_DEF_TIMO;
                                              *dev.ename  = '\0';
                                              *dev.file   = '\0'; }
        args '}'
;

args:   /* empty */
      | args NAME_TOKEN sep1 STR_TOKEN sep2 { strncpy( dev.name, $4,
                                                       NAME_MAX + 1 );
	                                      free( $4 ); }
      | args PAD_TOKEN sep1 NUM_TOKEN sep2  { dev.pad = $4; }
      | args SAD_TOKEN sep1 sad             { dev.sad = $4; }
      | args EOS_TOKEN sep1 NUM_TOKEN sep2  { dev.eos = $4; }
      | args TIMO_TOKEN sep1 NUM_TOKEN sep2 { dev.timo = $4; }
      | args REOS_TOKEN rt
      | args XEOS_TOKEN xt
      | args BIN_TOKEN bt
      | args EOT_TOKEN et
      | args MASTER_TOKEN mt
      | args FILE_TOKEN sep1 STR_TOKEN sep2 { strncpy( dev.file, $4,
                                                       PATH_MAX + 1 );
                                              free( $4 ); }
      | args ERR_TOKEN sep1 STR_TOKEN sep2  { strncpy( dev.ename, $4,
						       PATH_MAX + 1 );
                                              free( $4 ); }
;

sad:    NUM_TOKEN sep2                      { $$ = $1; }
      | NO_SAD_TOKEN sep2                   { $$ = NO_SAD; }
;

mt:     sep2                                { dev.flags |= IS_MASTER; }
      | sep1 bool sep2                      { if ( $2 )
	                                          dev.flags |= IS_MASTER;
                                              else
	                                          dev.flags &= ~ IS_MASTER; }
;

rt:     sep2                                { dev.eosmode |= GPIB_REOS; }
      | sep1 bool sep2                      { if ( $2 )
	                                          dev.eosmode |= GPIB_REOS;
                                              else
	                                          dev.eosmode &= ~ GPIB_REOS; }
;

xt:     sep2                                { dev.eosmode |= GPIB_XEOS; }
      | sep1 bool sep2                      { if ( $2 )
	                                          dev.eosmode |= GPIB_XEOS;
                                              else
	                                          dev.eosmode &= ~ GPIB_XEOS; }
;

bt:     sep2                                { dev.eosmode |= GPIB_BIN; }
      | sep1 bool sep2                      { if ( $2 )
	                                          dev.eosmode |= GPIB_BIN;
                                              else
	                                          dev.eosmode &= ~ GPIB_BIN; }
;

et:     sep2                                { dev.eosmode |= GPIB_EOT; }
      | sep1 bool sep2                      { if ( $2 )
	                                          dev.eosmode |= GPIB_EOT;
                                              else
	                                          dev.eosmode &= ~ GPIB_EOT; }
;

bool:   BOOL_TOKEN                          { $$ = $1; }
      | NUM_TOKEN                           { $$ = $1 ? 1 : 0; }
;

sep1:   /* empty */
      | '='
      | ':'
;

sep2:   /* empty */
      | ','
      | ';'
;


%%


void yyerror( const char * s )
{
	extern char *yytext;

	if ( gpib_error_buffer != NULL &&
	     gpib_error_buffer_length > 0 )
		snprintf( gpib_error_buffer, gpib_error_buffer_length,
			  "%s at token '%s' on line %d.\n",
			  s, yytext, yylineno );
    gpib_status |= GPIB_ERR;
}


/*
 * Local variables:
 * c-basic-offset: 8
 * c-indent-level: 8
 * c-brace-imaginary-offset: 0
 * c-brace-offset: 0
 * c-argdecl-indent: 4
 * c-label-ofset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: t
 * tab-width: 8
 * End:
 */
