# $Id: GPIB.pm 736 2006-07-07 15:48:51Z jens $
#
# Copyright (C) 2001-2006 Jens Thoms Toerring
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# To contact the author send email to: jt@toerring.de

package GPIB;

use 5.006;
use strict;
use warnings;
use Carp;

require Exporter;
require DynaLoader;

our @ISA = qw( Exporter DynaLoader );

our %EXPORT_TAGS = ( 'all' => [ qw( GPIB_ERR
                                    GPIB_TIMO
                                    GPIB_END
                                    GPIB_SRQI
                                    GPIB_RQS
                                    GPIB_CMPL
                                    GPIB_CIC
                                    GPIB_ATN
                                    GPIB_TACS
                                    GPIB_LACS
                                    GPIB_DTAS
                                    GPIB_DCAS

                                    EDVR
                                    ECIC
                                    ENOL
                                    EADR
                                    EARG
                                    ESAC
                                    EABO
                                    ENEB
                                    EDMA
                                    EBTO
                                    EOIP
                                    ECAP
                                    EFSO
                                    EOWN
                                    EBUS
                                    ESTB
                                    ESRQ
                                    ECFG
                                    EPAR
                                    ETAB
                                    ENSD
                                    ENWE
                                    ENTF
                                    EMEM
                                    ENMS

                                    TNONE
                                    T10us
                                    T30us
                                    T100us
                                    T300us
                                    T1ms
                                    T3ms
                                    T10ms
                                    T30ms
                                    T100ms
                                    T300ms
                                    T1s
                                    T3s
                                    T10s
                                    T30s
                                    T100s
                                    T300s
                                    T1000s

                                    REOS
                                    XEOS
                                    BIN
                                    EOT

                                    ASK_SAD
                                    ASK_PAD
                                    ASK_EOS
                                    ASK_XEOS
                                    ASK_REOS
                                    ASK_BIN
                                    ASK_EOT
                                    ASK_TIMO
                                    ASK_IS_INIT
                                    ASK_IS_MASTER
                                    ASK_IRQ
                                    ASK_DMA

                                    ValidDAV
                                    ValidNDAC
                                    ValidNRFD
                                    ValidIFC
                                    ValidREN
                                    ValidSRQ
                                    ValidATN
                                    ValidEOI

                                    BusDAV
                                    BusNDAC
                                    BusNRFD
                                    BusIFC
                                    BusREN
                                    BusSRQ
                                    BusATN
                                    BusEOI         ) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
);

our $VERSION = '0.01';

use constant GPIB_ERR      => 0x8000;
use constant GPIB_TIMO     => 0x4000;
use constant GPIB_END      => 0x2000;
use constant GPIB_SRQI     => 0x1000;
use constant GPIB_RQS      => 0x0800;
use constant GPIB_CMPL     => 0x0100;
use constant GPIB_CIC      => 0x0020;
use constant GPIB_ATN      => 0x0010;
use constant GPIB_TACS     => 0x0008;
use constant GPIB_LACS     => 0x0004;
use constant GPIB_DTAS     => 0x0002;
use constant GPIB_DCAS     => 0x0001;

use constant EDVR          =>  0;
use constant ECIC          =>  1;
use constant ENOL          =>  2;
use constant EADR          =>  3;
use constant EARG          =>  4;
use constant ESAC          =>  5;
use constant EABO          =>  6;
use constant ENEB          =>  7;
use constant EDMA          =>  8;
use constant EBTO          =>  9;
use constant EOIP          => 10;
use constant ECAP          => 11;
use constant EFSO          => 12;
use constant EOWN          => 13;
use constant EBUS          => 14;
use constant ESTB          => 15;
use constant ESRQ          => 16;
use constant ECFG          => 17;
use constant EPAR          => 18;
use constant ETAB          => 19;
use constant ENSD          => 20;
use constant ENWE          => 21;
use constant ENTF          => 22;
use constant EMEM          => 23;
use constant ENMS          => 24;

use constant TNONE         =>  0;
use constant T10us         =>  1;
use constant T30us         =>  2;
use constant T100us        =>  3;
use constant T300us        =>  4;
use constant T1ms          =>  5;
use constant T3ms          =>  6;
use constant T10ms         =>  7;
use constant T30ms         =>  8;
use constant T100ms        =>  9;
use constant T300ms        => 10;
use constant T1s           => 11;
use constant T3s           => 12;
use constant T10s          => 13;
use constant T30s          => 14;
use constant T100s         => 15;
use constant T300s         => 16;
use constant T1000s        => 17;

use constant REOS          => 1;
use constant XEOS          => 2;
use constant BIN           => 4;
use constant EOT           => 8;

use constant ASK_SAD       =>  1;
use constant ASK_PAD       =>  2;
use constant ASK_EOS       =>  3;
use constant ASK_XEOS      =>  4;
use constant ASK_REOS      =>  5;
use constant ASK_BIN       =>  6;
use constant ASK_EOT       =>  7;
use constant ASK_TIMO      =>  8;
use constant ASK_IS_INIT   =>  9;
use constant ASK_IS_MASTER => 10;
use constant ASK_IRQ       => 11;
use constant ASK_DMA       => 12;

use constant ValidDAV      => 0x01;
use constant ValidNDAC     => 0x02;
use constant ValidNRFD     => 0x04;
use constant ValidIFC      => 0x08;
use constant ValidREN      => 0x10;
use constant ValidSRQ      => 0x20;
use constant ValidATN      => 0x40;
use constant ValidEOI      => 0x80;

use constant BusDAV        => 0x01;
use constant BusNDAC       => 0x02;
use constant BusNRFD       => 0x04;
use constant BusIFC        => 0x08;
use constant BusREN        => 0x10;
use constant BusSRQ        => 0x20;
use constant BusATN        => 0x40;
use constant BusEOI        => 0x80;


bootstrap GPIB $VERSION;


########################################
#

sub new {

    my $inv = shift;
    my $pkg = ref( $inv ) || $inv;
    my $g = { };

    $g->{ name } = "unknown";
    if ( @_ == 1 ) {
        $g->{ dev } = gpib_find_xs( $_[ 0 ] );
        $g->{ name } = $_[ 0 ];
    } elsif ( @_ == 5 ) {
        $g->{ dev } = gpib_dev_xs( $_[ 0 ], $_[ 1 ], $_[ 2 ],
                                   $_[ 3 ], $_[ 4 ] );
        $g->{ name } = "unknown_" . $_[ 0 ] . "_" . $_[ 1 ];
    }

    die "GPIB->new failed for device \'$g->{ name }\'\n"
        unless defined $g->{ dev };

    bless $g, $pkg;

    $g->{ eos } = $g->ask( ASK_EOS );
    $g->{ eos_append } = 0;
    $g->{ eos_strip } = 0;
    $g->{ def_read_len } = 1024;

    return $g;
}


########################################
#

sub status {
    if ( @_ == 0 ) {
        gpib_global_status_xs( );
    } else {
        gpib_device_status_xs( shift->{ dev } );
    }
}


########################################
#

sub error {
    if ( @_ == 0 ) {
        gpib_global_error_xs( );
    } else {
        gpib_device_error_xs( shift->{ dev } );
    }
}


########################################
#

sub count {
    if ( @_ == 0 ) {
        gpib_global_count_xs( );
    } else {
        gpib_device_count_xs( shift->{ dev } );
    }
}


########################################
#

sub rsp {
    gpib_rsp_xs( shift->{ dev } );
}


########################################
#

sub read {
    my $g = shift;
    my $cnt = shift;

    $cnt = $g->{ def_read_len } unless defined $cnt;
    my $txt = gpib_rd_xs( $g->{ dev }, $cnt );
    $txt =~ s/$g->{ eos }$// if $g->{ eos } ne "\0" and $g->{ eos_strip };
    return wantarray ? ( $txt, $g->count, $g->status ) : $txt;
}


########################################
#

sub write {
    my $g = shift;
    my $buf = shift;
    $buf .= $g->{ eos } if $g->{ eos } ne "\0" and $g->{ eos_append };
    gpib_wrt_xs( $g->{ dev }, $buf );
    return wantarray ? ( $g->count, $g->status ) : $g->status;
}


########################################
#

sub command {
    shift if ( @_ == 2 );
    gpib_cmd( shift );
    return wantarray ? ( gpib_global_count_xs( ), gpib_global_status_xs( ) ) :
        gpib_global_status_xs( );
}


########################################
#

sub ask {
    my ( $g, $what ) = @_;

    my $ret = gpib_ask_xs( shift->{ dev }, shift );
    return chr( $ret ) if $what == ASK_EOS;
    return $ret;
}


########################################
#

sub clear {
    gpib_clr_xs( shift->{ dev } );
}


########################################
#

sub dma {
    return gpib_global_ask_xs( ASK_DMA ) if @_ == 0;

    my $state = shift;

    if ( ref( $state ) and ( $state->isa( 'GPIB' ) or $state eq 'GPIB' ) ) {
        if ( @_ == 0 ) {
            gpib_global_ask_xs( ASK_DMA );
        } else {
            gpib_dma_xs( shift );
        }
    } else {
        gpib_dma_xs( $state );
    }
}


########################################
#

sub irq {
    return gpib_global__ask_xs( ASK_IRQ ) if @_ == 0;

    my $state = shift;

    if ( ref( $state ) and ( $state->isa( 'GPIB' ) or $state eq 'GPIB' ) ) {
        if ( @_ == 0 ) {
            gpib_global_ask_xs( ASK_IRQ );
        } else {
            gpib_irq_xs( shift );
        }
    } else {
        gpib_irq_xs( $state );
    }
}


########################################
#

sub eos_char {
    my $g = shift;

    if ( @_ == 0 ) {
        chr( gpib_ask_xs( $g->{ dev }, ASK_EOS ) );
    } else {
        
        my $eos = shift;
        croak "Invalid EOS character" if ( $eos < 0 or $eos > 255 );
        gpib_eos_char_xs( $g->{ dev }, $eos );
        $g->{ eos } = $eos unless g->status & GPIB_ERR;
    }
}


########################################
#

sub set_eos_binary_compare {
    my $dev = shift->{ dev };

    return gpib_ask_xs( $dev, ASK_BIN ) if @_ == 0;

    my $eos_mode = 0;
    $eos_mode |= BIN if $_[ 0 ];
    $eos_mode |= REOS if gpib_ask_xs( $dev, ASK_REOS );
    $eos_mode |= XEOS if gpib_ask_xs( $dev, ASK_XEOS );
    $eos_mode |= EOT  if gpib_ask_xs( $dev, ASK_EOT );

    gpib_eos_mode_xs( $dev, $eos_mode );
}


########################################
#

sub terminate_read_on_eos {
    my $dev = shift->{ dev };

    return gpib_ask_xs( $dev, ASK_REOS ) if @_ == 0;

    my $eos_mode = 0;
    $eos_mode |= REOS if $_[ 0 ];
    $eos_mode |= BIN  if gpib_ask_xs( $dev, ASK_BIN );
    $eos_mode |= XEOS if gpib_ask_xs( $dev, ASK_XEOS );
    $eos_mode |= EOT  if gpib_ask_xs( $dev, ASK_EOT );

    gpib_eos_mode_xs( $dev, $eos_mode );
}


########################################
#

sub raise_eoi_with_eos_on_write {
    my $dev = shift->{ dev };

    return gpib_ask_xs( $dev, ASK_XEOS ) if @_ == 0;

    my $eos_mode = 0;
    $eos_mode |= XEOS if $_[ 0 ];
    $eos_mode |= BIN  if gpib_ask_xs( $dev, ASK_BIN );
    $eos_mode |= REOS if gpib_ask_xs( $dev, ASK_REOS );
    $eos_mode |= EOT  if gpib_ask_xs( $dev, ASK_EOT );

    gpib_eos_mode_xs( $dev, $eos_mode );
}


########################################
#

sub default_read_length {
    my $g = shift;

    return $g->{ def_read_len } if @_ == 0;

    my $len = shift;
    croak "Invalid default read length" unless $len =~ /^\d+$/o and $len > 0;
    $g->{ def_read_len } = $len;
}


########################################
#

sub automatic_eos_append {
    my $g = shift;

    if ( @_ >= 1 ) {
        $g->{ eos_append } = shift;
    } else {
        return $g->{ eos_append };
    }
}


########################################
#

sub automatic_eos_strip {
    my $g = shift;

    if ( @_ >= 1 ) {
        $g->{ eos_strip } = shift;
    } else {
        return $g->{ eos_strip };
    }
}


########################################
#

sub raise_eoi_with_last_byte {
    my $dev = shift->{ dev };

    return gpib_ask_xs( $dev, ASK_EOT ) if @_ == 0;

    my $eos_mode = 0;
    $eos_mode |= EOT if $_[ 0 ];
    $eos_mode |= BIN  if gpib_ask_xs( $dev, ASK_BIN );
    $eos_mode |= REOS if gpib_ask_xs( $dev, ASK_REOS );
    $eos_mode |= XEOS if gpib_ask_xs( $dev, ASK_XEOS );

    gpib_eos_mode_xs( $dev, $eos_mode );
}


########################################
#

sub eos_mode {
    gpib_eos_mode_xs( shift->{ dev }, shift );
}


########################################
#

sub gts {
    gpib_gts_xs( );
}


########################################
#

sub lines {
    my $lines = gpib_lines_xs( );
    return wantarray ? ( $lines, gpib_global_status_xs( ) ) : $lines;
}


########################################
#

sub wait {
    my $mask = shift;

    if ( ref( $mask ) and ( $mask->isa( 'GPIB' ) or $mask eq 'GPIB' ) ) {
        if ( $mask->isa( 'GPIB' ) ) {
            gpib_wait_xs( $mask->{ dev }, shift );
        } else {
            gpib_wait_xs( 0, shift );
        }
    } else {
        gpib_wait_xs( 0, $mask );
    }
}


########################################
#

sub ppc {
    gpib_ppc_xs( shift->{ dev }, shift );
}


########################################
#

sub rpp {
    gpib_rpp_xs( );
}


########################################
#

sub local {
    gpib_loc_xs( shift->{ dev } );
}


########################################
#

sub online {
    if ( @_ == 1 ) {
        gpib_ask_xs( shift->{ dev }, ASK_IS_INIT );
    } else {
        gpib_onl_xs( shift->{ dev }, shift );
    }
}


########################################
#

sub primary_address {
    if ( @_ == 1 ) {
        gpib_ask_xs( shift->{ dev }, ASK_PAD );
    } else {
        croak "Invalid primary address" if ( $_[ 1 ] < 0 or $_[ 1 ] > 30 );
        gpib_pad_xs( shift->{ dev }, shift );
    }
}


########################################
#

sub secondary_address {
    if ( @_ == 1 ) {
        gpib_ask_xs( shift->{ dev }, ASK_SAD );
    } else {
        croak "Invalid secondary address" if ( $_[ 1 ] < 96 or $_[ 1 ] > 126 );
        gpib_sad_xs( shift->{ dev }, shift );
    }
}


########################################
#

sub sre {
    my $state = shift;
    gpib_sre_xs( ref( $state ) and
                 ( $state->isa( 'GPIB' ) or $state eq 'GPIB' ) ?
                 shift : $state );
}


########################################
#

sub timeout {

    if ( @_ == 1 ) {
        gpib_ask_xs( shift->{ dev }, ASK_TIMO );
    } else { 
        croak "Invalid timeout argument"
            if ( $_[ 1 ] < TNONE or $_[ 1 ] > T1000s );
        gpib_tmo_xs( shift->{ dev }, shift );
    }
}


########################################
#

sub trigger {
    gpib_trg_xs( shift->{ dev } );
}


########################################
#

sub dcl {
    gpib_dcl_xs(  );
}


########################################
#

sub ln {
    my $g = shift;

    if ( ref( $g ) and ( $g->isa( 'GPIB' ) or $g eq 'GPIB' ) ) {
        if ( $g eq 'GPIB' ) {
            return gpib_ln_xs( shift, shift );
        } else {
            return gpib_ln_xs( gpib_ask_xs( $g->{ dev }, ASK_PAD ),
                               gpib_ask_xs( $g->{ dev }, ASK_SAD ) );
        }
    } else {
        return gpib_ln_xs( $g, shift );
    }
}


########################################
#

sub name {
    shift->{ name };
}


########################################
#

sub status_list {

    my $s = 0;
    my $l = "";

    if ( @_ == 0 ) {
        $s = gpib_global_status_xs( 0 );
    } else {
        $s = gpib_device_status_xs( shift->{ dev } );
    }

    $l .= "ERR "  if ( $s & GPIB_ERR );
    $l .= "TIMO " if ( $s & GPIB_TIMO );
    $l .= "END "  if ( $s & GPIB_END );
    $l .= "SRQI " if ( $s & GPIB_SRQI );
    $l .= "RQS "  if ( $s & GPIB_RQS );
    $l .= "CMPL " if ( $s & GPIB_CMPL );
    $l .= "CIC "  if ( $s & GPIB_CIC );
    $l .= "ATN "  if ( $s & GPIB_ATN );
    $l .= "TACS " if ( $s & GPIB_TACS );
    $l .= "LACS " if ( $s & GPIB_LACS );
    $l .= "DTAS " if ( $s & GPIB_DTAS );
    $l .= "DCAS " if ( $s & GPIB_DCAS );

    chop $l if $l ne "";
    return $l;
}


########################################
#

sub error_list {

    my $e;

    if ( @_ == 0 ) {
        $e = gpib_global_error_xs( );
    } else {
        $e = gpib_device_error_xs( shift->{ dev } );
    }

    return "" if $e == 0;

    return "EDVR System error"                      if $e == EDVR;
    return "ECIC Board not CIC"                     if $e == ECIC;
    return "ENOL No listener"                       if $e == ENOL;
    return "EADR Board not addressed"               if $e == EADR;
    return "EARG Invalid arguments"                 if $e == EARG;
    return "ESAC Board not system controller"       if $e == ESAC;
    return "EABO I/O operation aborted"             if $e == EABO;
    return "ENEB Nonexistent board"                 if $e == ENEB;
    return "EDMA DMA error"                         if $e == EDMA;
    return "EOIP Asynchronous I/O in progress"      if $e == EOIP;
    return "ECAP No capability for operation"       if $e == ECAP;
    return "EFSO File system error"                 if $e == EFSO;
    return "EBUS GPIB bus error"                    if $e == EBUS;
    return "ESTB Serial poll status queue overflow" if $e == ESTB;
    return "ESRQ SRQ stuck in ON position"          if $e == ESRQ;
    return "ECFG Unable to open configuration file" if $e == ECFG;
    return "EPAR Invalid configuration file"        if $e == EPAR;
    return "EMEM Not enough memory"                 if $e == EMEM;
}


1;
__END__

=head1 NAME

GPIB - Perl interface for the GPIB library

=head1 SYNOPSIS

  use GPIB;

  $dev = GPIB->new( 'device_name' );
  $dev->write( 'command_string' );
  $reply = $dev->read;

=head1 DESCRIPTION

This package is interface to the GPIB library for Perl. It allows to use the
library functions in an object oriented manner.

To use the library you will need to install a configuration file for
the library. This file is typically /etc/gpib.conf, but by setting
the environment variable GPIB_CONF a different file can be used.
Please see the documentation for the library for information on the
syntax of the configuration file.

When the configuration file exists it defines a unique name for every
device. If one of the devices has, for example, the name "TDS520A",
use

    use GPIB;

    $dev = GPIB->new( 'TDS520A' );

to create a new device object referring to this device. Alternatively,
for devices not listed in the configuration file one may use the B<new>
method with 5 arguments:

    $dev = GPIB->new( $pad, $sad, $timo, $eos_char, $eos_mode )

where $pad and $pad stand for the primary and secondary GPIB address of
the device, $timo is the timeout index value and $eos_char and $eos_mode
are the EOS character and the EOS detection mode.

After having obtained a device object data can be written to the device
by employing the B<write> method with the string to be send and data be
read using the B<read> method. E.g. use

    $dev->write( 'HORIZONTAL:MAIN:SCALE?\n' );
    $reply = $dev->read;

The variable $reply should now contain the string the device sent back. You
may pass an argument to the B<read> method with the maximum number of
characters to be read. When this argument is not given B<read> defaults to a
maximum number of characters that can be set individually for each device.

The following methods and functions are supported by the package:

=over 4

=item B<write>

Sends a string to the device. It expects one argument, the string to be
transfered to the device. when called in scalar context the B<write>
method returns the status resulting from the command, otherwise an
array with two elements, the number of characters send to the device
and the status.

=item B<read>

Reads a string from the device. It accepts one optional argument, the
maximum length of the string to be read. If not given a default value
is used, that initially is 1024 characters (but can be changed via the
B<default_read_length> function for the individual devices). When called
in scalar context B<read> returns the string from the device, otherwise
an array with three elements, the string send by the device, the number
of characters read and the status.

=item B<status>

When used as a method (i.e. via "$dev->status") it returns the
status byte from the last method call for the device. If called as
a function without an argument (i.e. via "GPIB::status") the current
value of the status byte is returned.

=item B<error>

When used as a method (i.e. via "$dev->error") it returns the
error byte from the last method call for the device. If called as
a function without an argument (i.e. via "GPIB::error") the current
value of the error byte is returned.

=item B<count>

When used as a method (i.e. via "$dev->count") it returns the
the number of characters read or written by the last call of the
B<read>, B<write> or B<command> method for the device. If falled
as a function with no arguments (i.e. via "GPIB::count") the count
result from the last GPIB method or function call is returned.

=item B<wait>

Not yet documented.

=item B<ppc>

Not yet documented.

=item B<rpp>

Not yet documented.

=item B<clear>

Sends the device a Selected Device Clear (SDC) message.

=item B<trigger>

Sends the device a Group Execute Trigger (GET) message.

=item B<timeout>

When called without an argument the method returns the timeout value used
when reading or writing data to the device. If called with an argument
it sets a new timeout value. Timeout values are integers between 0 and 17
(TNONE to T1000s) indexing a timeout table, see the library documentation
for more information.

=item B<eos_char>

When called with no argument the method returns the EOS character currently
used when transferring data to and from the device. An EOS character of
0 indicates that no EOS character is used. When called with an argument
(between 0 and 255) a new EOS character is set for the device.

=item B<set_eos_binary_compare>

When called with no argument a non-zero return value indicates that 
the comparison of received characters with EOS character is done using all
8 bits, otherwise the lower 7 bits only are used. If called with an argument
the argument determines if on further reads an 8 bit or a 7 bit only
comparison is done.

=item B<terminate_read_on_eos>

When called with no argument the method returns a value that indicates if
read operations are terminated on receipt of the EOS character (if the
returned value is non-zero) or not. I called with a non-zero argument
further read operations from the device are termimated on receipt of the
EOS character, if called with a zero argument only a raised EOI line is
taken to indicate the end of a message.

=item B<raise_eoi_with_eos_on_write>

When called with no argument a non-zero return value indicates that the
EOI line is raised with each EOS character transmitted to the device.
If called with a non-zero argument on futher write operation to the
device the EOI line will be raised together with each EOS character
in the data.

=item B<raise_eoi_with_last_byte>

When called with no argument a non-zero return value indicates that the
EOI line is raised while sending the last character of a message. If called
with a non-zero argument the EOI line will be raised together with the last
character of a message on further write operations.

=item B<automatic_eos_append>

If this method is applied to a device object with a true argument in further
B<write> operations the EOS character defined for the device will
automatically be appended to the string to be send to the device. By calling
the method with a false argument this can be switched off again (which is the
default behaviour for a newly created device object). When invoked with no
argument the current state for the device is returned.

=item B<automatic_eos_strip>

If this method is applied to a device object with a true argument in further
B<read> operations the EOS character will be removed automatically from the
end of string received from the device (if there is one). Calling the method
with a false argument switches this off (which is the default behaviour for a
newly created device object). When invoked with no argument the current state
for the device is returned.

=item B<default_read_length>

When B<read>ing from a device without specifying the maximum number of
characters to be read a default value of 1024 characters is used. With this
method the default value can be set to a different value for each device.

=item B<primary_address>

When called with no argument the function returns the primary GPIB address of
the device. Otherwise it sets a new primary GPIB address of the device to the
value of the argument. Primary GPIB address must be in the range between 0 and
30.

=item B<secondary_address>

When called with no argument the function returns the secondary GPIB address
of the device. If called with an argument it sets a new secondary GPIB address
for the device. Secondary GPIB address must be in the range between 96 and
126. A secondary address of 0 indicates that no secondary addresssing is to
be used.

=item B<command>

Not yet documented.

=item B<ask>

Not yet documented.

=item B<irq>

If called with no argument the method returns a value indicating if
interrupts is used in reading and writing. When called with an argument
the use of interrupts is switched on or off depending on the argument.
Please note that switching off the use of interrupts also switches off
DMA.

=item B<dma>

If called with no argument the method returns a value indicating if DMA is
used for reading and writing. When called with an argument the use of DMA is
switched on or off depending on the argument. Please note that DMA can not be
used if no interrupts are used.

=item B<lines>

This method returns the current state of the GPIB lines. The result is an
integer of two bytes, where the upper byte represents the status of the GPIB
lines while the bits in the lower byte indicate if the values of the bits in
the upper byte can be trusted. Please note that this method does not work
for all GPIB boards and in this case the lower byte is set to 0.

=item B<gts>

Not yet documented.

=item B<loc>

Not yet documented.

=item B<online>

Not yet documented.

=item B<sre>

Not yet documented.

=back

=head2 EXPORT

None by default.


=head1 CREDIT

This package is loosely based on a Perl GPIB package by Jeff Mock
<jeff@mock.com>.


=head1 AUTHOR

Jens Thoms Trring, <jt@toerring.de>


=head1 SEE ALSO

Documentation for the GPIB library.

=cut


# Local variables:
# indent-tabs-mode: nil
# tab-width: 8
# End:
