/*
   $Id: GPIB.xs 736 2006-07-07 15:48:51Z jens $
   
   Copyright (C) 2001-2006 Jens Thoms Toerring
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   
   To contact the author send email to: jt@toerring.de
*/


#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
#include "gpib.h"


typedef struct {
    int     dev_dscr;
    int     status;
    int     error;
    unsigned long count;
} GPIB_struct;

#define CONTROLLER 0

MODULE = GPIB       PACKAGE = GPIB


#####################################

GPIB_struct *
gpib_find_xs( name )
    const char *name

    PREINIT:
        GPIB_struct *g;

    CODE:
        New( 0, g, 1, GPIB_struct );
        g->dev_dscr = gpib_find( name );
        if ( g->dev_dscr >= 0 ) {
            g->status = gpib_status;
            g->error  = gpib_error;
            g->count  = gpib_count;
            RETVAL = g;
        } else {
            safefree( g );
            RETVAL = NULL;
        }

    OUTPUT:
        RETVAL


#####################################

GPIB_struct *
gpib_dev_xs( pad, sad, timo, eos_char, eos_mode )
    int pad
    int sad
    int timo
    char eos_char
    int eos_mode

    PREINIT:
        GPIB_struct *g;

    CODE:
        New( 0, g, 1, GPIB_struct );
        g->dev_dscr = gpib_dev( pad, sad, timo, eos_char, eos_mode );
        if ( g->dev_dscr >= 0 ) {
            g->status = gpib_status;
            g->error  = gpib_error;
            g->count  = gpib_count;
            RETVAL = g;
        } else {
            safefree( g );
            RETVAL = NULL;
        }

    OUTPUT:
        RETVAL


#####################################

SV *
gpib_DESTROY( g )
    GPIB_struct *g

    CODE:
        printf( "Destroying %d\n", g->dev_dscr );
        safefree( g );


#####################################

int
gpib_wrt_xs( g, buffer )
    GPIB_struct *g
    const char *buffer

    CODE:
        RETVAL = gpib_wrt( g->dev_dscr, buffer, strlen( buffer ) );
        g->count  = gpib_count;
        g->status = gpib_status;
        g->error  = gpib_error;

    OUTPUT:
        RETVAL


#####################################

SV *
gpib_rd_xs( g, count )
    GPIB_struct *g
    size_t count

    PREINIT:
        char *buffer;
        int ret;

    CODE:
        New( 0, buffer, count + 1, char );
        ST( 0 ) = sv_newmortal( );
        ret = gpib_rd( g->dev_dscr, buffer, count );
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

        if ( gpib_count > 0 )
        {
            buffer[ gpib_count ] = '\0';
            sv_setpv( ST( 0 ), buffer );
        }
        else {
            ST( 0 ) = &PL_sv_undef;
        }

        safefree( buffer );


#####################################

int
gpib_cmd_xs( buffer )
    const char *buffer

    CODE:
        RETVAL = gpib_cmd( buffer, strlen( buffer ) );

    OUTPUT:
        RETVAL


#####################################

int
gpib_ask_xs( g, option )
    GPIB_struct *g
    int option

    PREINIT:
        int value;

    CODE:
        gpib_ask( g->dev_dscr, option, &value );
        g->count  = gpib_count;
        g->status = gpib_status;
        g->error  = gpib_error;
        RETVAL = value;

    OUTPUT:
        RETVAL


#####################################

int
gpib_global_ask_xs( option)
    int option

    PREINIT:
        int value;

    CODE:
        gpib_ask( CONTROLLER, option, &value );
        RETVAL = value;

    OUTPUT:
        RETVAL


#####################################

int
gpib_tmo_xs( g, tmo )
    GPIB_struct *g
    int tmo

    CODE:
        RETVAL = gpib_tmo( g->dev_dscr, tmo );
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

    OUTPUT:
        RETVAL


#####################################

int
gpib_pad_xs( g, pad )
    GPIB_struct *g
    int pad

    CODE:
        RETVAL = gpib_pad( g->dev_dscr, pad );
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

    OUTPUT:
        RETVAL


#####################################

int
gpib_sad_xs( g, sad )
    GPIB_struct *g
    int sad

    CODE:
        RETVAL = gpib_sad( g->dev_dscr, sad );
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

    OUTPUT:
        RETVAL


#####################################

int 
gpib_loc_xs( g )
    GPIB_struct *g

    CODE:
        RETVAL = gpib_loc( g->dev_dscr );
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

    OUTPUT:
        RETVAL


#####################################

int
gpib_trg_xs( g )
    GPIB_struct *g

    CODE:
        RETVAL = gpib_trg( g->dev_dscr );
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

    OUTPUT:
        RETVAL


#####################################

int
gpib_clr_xs( g )
    GPIB_struct *g

    CODE:
        RETVAL = gpib_clr( g->dev_dscr );
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

    OUTPUT:
        RETVAL


#####################################

int
gpib_rsp_xs( g )
    GPIB_struct *g

    PREINIT:
        unsigned char status_byte;

    CODE:
        gpib_rsp( g->dev_dscr, &status_byte );
        RETVAL = ( int ) status_byte;
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

    OUTPUT:
        RETVAL


#####################################

int
gpib_eos_xs( g, eos_byte )
    GPIB_struct *g
    int eos_byte

    CODE:
        RETVAL = gpib_eos( g->dev_dscr, eos_byte );
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

    OUTPUT:
        RETVAL


#####################################

int
gpib_onl_xs( g, v )
    GPIB_struct *g
    int v

    CODE:
        RETVAL = gpib_onl( g->dev_dscr, v );
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

    OUTPUT:
        RETVAL


#####################################

int
gpib_sre_xs( v )
    int v

    CODE:
        RETVAL = gpib_sre( v );

    OUTPUT:
        RETVAL


#####################################

int
gpib_gts_xs( )

    CODE:
        RETVAL = gpib_gts( );

    OUTPUT:
        RETVAL


#####################################

int
gpib_dcl_xs( )

    CODE:
        RETVAL = gpib_dcl( );

    OUTPUT:
        RETVAL


#####################################

int
gpib_irq_xs( state )
    int state

    CODE:
        RETVAL = gpib_irq( state );

    OUTPUT:
        RETVAL


#####################################

int
gpib_dma_xs( state )
    int state

    CODE:
        RETVAL = gpib_dma( state );

    OUTPUT:
        RETVAL


#####################################

int
gpib_lines_xs( )

    PREINIT:
        int lines;

    CODE:
        gpib_lines( &lines );
        RETVAL = lines;

    OUTPUT:
        RETVAL


#####################################

int
gpib_wait_xs( g, v )
    GPIB_struct *g
    int v

    CODE:
        RETVAL = gpib_wait( v );
        if ( g != NULL )
        {
            g->status = gpib_status;
            g->error  = gpib_error;
        }

    OUTPUT:
        RETVAL



#####################################

int
gpib_ppc_xs( g, mask )
    GPIB_struct *g
    int mask

    CODE:
        RETVAL = gpib_ppc( g->dev_dscr, mask );
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

    OUTPUT:
        RETVAL


#####################################

int
gpib_rpp_xs( )

    PREINIT:
        char ppr = 0;

    CODE:
        gpib_rpp( &ppr );
        RETVAL = ( int ) ppr;

    OUTPUT:
        RETVAL


#####################################

int 
gpib_sic_xs( )

    CODE:
        RETVAL = gpib_sic( );

    OUTPUT:
        RETVAL


#####################################

int
gpib_eos_char_xs( g, eos_char )
    GPIB_struct *g
    int eos_char

    CODE:
        RETVAL = gpib_eos_char( g->dev_dscr, eos_char );
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

    OUTPUT:
        RETVAL


#####################################

int
gpib_eos_mode_xs( g, eos_mode )
    GPIB_struct *g
    int eos_mode

    CODE:
        RETVAL = gpib_eos_mode( g->dev_dscr, eos_mode );
        g->status = gpib_status;
        g->error  = gpib_error;
        g->count  = gpib_count;

    OUTPUT:
        RETVAL

    
#####################################

int
gpib_ln_xs( pad, sad )
    int pad
    int sad

    PREINIT:
        short int listener;

    CODE:
        gpib_ln( pad, sad, &listener );
        RETVAL = ( int ) listener;

    OUTPUT:
        RETVAL


#####################################

int
gpib_device_status_xs( g )
    GPIB_struct *g

    CODE:
        if ( g == NULL )
            RETVAL = gpib_status;
        else
            RETVAL = g->status;

    OUTPUT:
        RETVAL


#####################################

int
gpib_global_status_xs( )

    CODE:
        RETVAL = gpib_status;

    OUTPUT:
        RETVAL


#####################################

int
gpib_device_error_xs( g )
    GPIB_struct *g

    CODE:
        RETVAL = g->error;

    OUTPUT:
        RETVAL


#####################################

int
gpib_global_error_xs( )

    CODE:
        RETVAL = gpib_error;

    OUTPUT:
        RETVAL


#####################################

int
gpib_device_count_xs( g )
    GPIB_struct *g

    CODE:
        RETVAL = g->count;

    OUTPUT:
        RETVAL


#####################################

int
gpib_global_count_xs( )

    CODE:
        RETVAL = gpib_count;

    OUTPUT:
        RETVAL


/*
 * Local variables:
 * indent-tabs-mode: nil
 * tab-width: 4
 * End:
 */
